@@ -2,16 +2,16 @@
 // Copyright (C) 2018, 2021, 2022 Soni L.
 //
 // This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU Affero General Public License as
+// it under the terms of the GNU General Public License as
 // published by the Free Software Foundation, either version 3 of the
 // License, or (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU Affero General Public License for more details.
+// GNU General Public License for more details.
 //
-// You should have received a copy of the GNU Affero General Public License
+// You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 //! Write hexchat plugins in Rust!
@@ -148,7 +148,7 @@
  *         -[x] libdirfs
  *         -[ ] gtkwin_ptr
  *         -[ ] win_ptr
- *     -[ ] hexchat_get_prefs
+ *     -[x] hexchat_get_prefs
  *     -[ ] hexchat_list_get
  *     -[ ] hexchat_list_fields
  *     -[ ] hexchat_list_next
@@ -328,6 +328,14 @@ pub struct PluginHandle<'ph> {
     info: PluginInfo,
 }
 
+/// A setting value, returned by [`ValidContext::get_prefs`].
+#[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug)]
+pub enum PrefValue {
+    String(String),
+    Int(i32),
+    Bool(bool),
+}
+
 mod valid_context {
     use crate::PluginHandle;
 
@@ -403,11 +411,68 @@ pub struct InvalidContextError<F>(F);
 // ***** //
 
 impl<F> InvalidContextError<F> {
+    /// Returns the closure wrapped within this error.
     pub fn get_closure(self) -> F {
         self.0
     }
 }
 
+impl_trait! {
+    impl PrefValue {
+        /// Projects this `PrefValue` as `&str`, if it is a `String`. Returns
+        /// `None` otherwise.
+        pub fn as_str(&self) -> Option<&str> {
+            match self {
+                &Self::String(ref s) => Some(s),
+                _ => None,
+            }
+        }
+
+        /// Projects this `PrefValue` as `i32`, if it is an `Int`. Returns
+        /// `None` otherwise.
+        pub fn as_i32(&self) -> Option<i32> {
+            match self {
+                &Self::Int(i) => Some(i),
+                _ => None,
+            }
+        }
+
+        /// Projects this `PrefValue` as `bool`, if it is a `Bool`. Returns
+        /// `None` otherwise.
+        pub fn as_bool(&self) -> Option<bool> {
+            match self {
+                &Self::Bool(b) => Some(b),
+                _ => None,
+            }
+        }
+
+        /// Consumes this `PrefValue` and attempts to take a `String` out of
+        /// it.
+        pub fn into_string(self) -> Option<String> {
+            match self {
+                Self::String(s) => Some(s),
+                _ => None,
+            }
+        }
+
+        impl trait From<String> {
+            fn from(s: String) -> Self {
+                Self::String(s)
+            }
+        }
+        impl trait From<i32> {
+            fn from(i: i32) -> Self {
+                Self::Int(i)
+            }
+        }
+        impl trait From<bool> {
+            fn from(b: bool) -> Self {
+                Self::Bool(b)
+            }
+        }
+    }
+}
+
 impl<'ph, 'f> HookHandle<'ph, 'f> where 'f: 'ph {
     /// If this is a timer hook, returns whether the hook has expired.
     ///
@@ -1187,6 +1252,9 @@ impl<'ph> PluginHandle<'ph> {
         String::from_utf8(out).unwrap()
     }
 
+    // TODO pluginprefs but see: https://developer.gimp.org/api/2.0/glib/glib-String-Utility-Functions.html#g-strescape
+    // var.len() + g_strescape(value).len() + 3 < 512
+
     // ******* //
     // PRIVATE //
     // ******* //
@@ -1212,6 +1280,7 @@ impl<'ph> PluginHandle<'ph> {
 }
 
 impl<'a> EventAttrs<'a> {
+    /// Creates a new `EventAttrs`.
     pub fn new() -> EventAttrs<'a> {
         EventAttrs {
             server_time: None,
@@ -1271,6 +1340,15 @@ impl<'a, 'ph: 'a> ValidContext<'a, 'ph> {
     }
 
     /// Compares two nicks based on the server's case mapping.
+    ///
+    /// ```no_run
+    /// use hexchat_unsafe_plugin::ValidContext;
+    ///
+    /// /// Checks if the two nicks below are equal.
+    /// fn compare_nicks(context: ValidContext<'_, '_>) -> bool {
+    ///     context.nickcmp("hello", "HELLO").is_eq()
+    /// }
+    /// ```
     pub fn nickcmp(&self, nick1: &str, nick2: &str) -> std::cmp::Ordering {
         use std::cmp::Ordering;
         let ph = &self.ph;
@@ -1289,6 +1367,7 @@ impl<'a, 'ph: 'a> ValidContext<'a, 'ph> {
         }
     }
 
+    /// Sends a list of modes to the current channel.
     pub fn send_modes<'b, I: IntoIterator<Item=&'b str>>(&mut self, iter: I, mpl: i32, sign: char, mode: char) {
         let ph = &self.ph;
         assert!(sign == '+' || sign == '-', "sign must be + or -");
@@ -1299,7 +1378,43 @@ impl<'a, 'ph: 'a> ValidContext<'a, 'ph> {
         let arr: &mut [*const c_char] = &mut *v2;
         unsafe {
             ph_call!(hexchat_send_modes(ph, arr.as_mut_ptr(), arr.len() as c_int,
-                mpl as c_int, sign as c_char, mode as c_char))
+                mpl as c_int, sign as c_char, mode as c_char));
+        }
+        // some hexchat forks may invalidate the context here.
+        // just pretend everything is fine and don't bump the major version.
+        if !self.set_context(&self.get_context()) {
+            let context = self.ph.find_valid_context().unwrap();
+            if !self.set_context(&context) { panic!(); }
+        }
+    }
+
+    /// Returns a client setting.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// use hexchat_unsafe_plugin::ValidContext;
+    ///
+    /// /// Returns the user's configured main nick.
+    /// fn main_nick(context: ValidContext<'_, '_>) -> String {
+    ///     context.get_prefs("irc_nick1").unwrap().into_string().unwrap()
+    /// }
+    /// ```
+    pub fn get_prefs(&self, name: &str) -> Option<PrefValue> {
+        let ph = &self.ph;
+        let name = CString::new(name).unwrap();
+        let mut string = 0 as *const c_char;
+        let mut int: c_int = 0;
+        match unsafe {
+            ph_call!(hexchat_get_prefs(ph, name.as_ptr(), &mut string, &mut int))
+        } {
+            0 => None,
+            1 => Some(PrefValue::String(unsafe {
+                CStr::from_ptr(string).to_owned().into_string().unwrap()
+            })),
+            2 => Some(PrefValue::Int(int as _)),
+            3 => Some(PrefValue::Bool(int != 0)),
+            _ => panic!("unsupported type in get_prefs"),
         }
     }
 
